/*
 * This file is part of Skyward Hub.
 *
 * Skyward Hub is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * Skyward Hub is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * Skyward Hub. If not, see <https://www.gnu.org/licenses/>.
 *
 */

#include "Module.h"
// #include <Components/ErrorDisplayer/Error.h>

#include <Components/ModulesPicker/ModulesPicker.h>
#include <Core/SkywardHubCore.h>
#include <Modules/ModulesList.h>

#include <QAction>

Module::Module(ModuleId id) : id(id)
{
    // Setup context menu
    {
        setContextMenuPolicy(Qt::CustomContextMenu);
        connect(this, &QWidget::customContextMenuRequested, this,
                [&](const QPoint &pos)
                {
                    QMenu menu("App");

                    QAction *close = new QAction("Close");
                    connect(close, &QAction::triggered, this,
                            [this]() { emit closeMe(this); });
                    menu.addAction(close);

                    QAction *replace = new QAction("Replace");
                    connect(replace, &QAction::triggered, this,
                            &Module::onReplaceMe);
                    menu.addAction(replace);

                    // Load custom actions
                    if (customContextMenuActions.size() > 0)
                    {
                        menu.addSeparator();
                        menu.addActions(customContextMenuActions);
                    }

                    // Load hub actions
                    menu.addSeparator();

                    SkywardHubCore::getInstance().setupMenu(menu);

                    menu.exec(mapToGlobal(pos));
                });
    }
}

ModuleId Module::getId() const { return id; }

QString Module::getModuleName() const
{
    return ModulesList::getInstance().getModuleName(id);
};

QString Module::getXmlName(const ModuleId id) const
{
    return ModulesList::getInstance().getXmlName(id);
};

XmlObject Module::toXmlObject() { return XmlObject(getXmlName(id)); }

void Module::fromXmlObject(const XmlObject &xmlObject) {}

void Module::onReplaceMe()
{
    ModulesPicker *picker = new ModulesPicker();

    connect(picker, &ModulesPicker::onModuleSelected, this,
            [this](ModuleId id) {
                emit replaceMe(
                    this, ModulesList::getInstance().instantiateModule(id));
            });

    // Launch the modules picker
    picker->exec();
}

Window *Module::getWindow()
{
    QObject *parent = this;
    while (dynamic_cast<Window *>(parent) == NULL)
    {
        parent = parent->parent();
    }
    return dynamic_cast<Window *>(parent);
}

void Module::error(QString title, QString description, int msecDuration)
{
    getWindow()->errorDisplayer->createError(title, description,
                                             ErrorType::ET_ERROR, msecDuration);
}

void Module::warning(QString title, QString description, int msecDuration)
{
    getWindow()->errorDisplayer->createError(
        title, description, ErrorType::ET_WARNING, msecDuration);
}

void Module::info(QString title, QString description, int msecDuration)
{
    getWindow()->errorDisplayer->createError(title, description,
                                             ErrorType::ET_INFO, msecDuration);
}
