/*
 * This file is part of Skyward Hub.
 *
 * Skyward Hub is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * Skyward Hub is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * Skyward Hub. If not, see <https://www.gnu.org/licenses/>.
 *
 */

#pragma once

#include <QStringList>

class Module;

enum ModuleId
{
    EMPTY,

    // Data visualization
    GRAPH,
    OUTGOING_MESSAGES_VIEWER,
    INCOMING_MESSAGES_VIEWER,
    MAIN_STATE_VIEWER,
    PAYLOAD_STATE_VIEWER,
    RIG_STATE_VIEWER,
    ORIENTATION_VISUALIZER,
    VALVES_VIEWER,

    // Data sources
    SERIAL_MAVLINK,
    UDP_MAVLINK,
    FILE_STREAM,

    // Utilities
    COMMAND_PAD,
    COMPACT_COMMAND_PAD,
    VALUES_CONVERTER_VIEWER,
    TIMER_CONTROLLER,

    // Organization
    TABS,
    VERTICAL_SPLITTER,
    HORIZONTAL_SPLITTER,

    // Testing
    BROKER_TEST

};

enum ModuleCategory
{
    HIDDEN,
    DATA_VISUALIZATION,
    DATA_SOURCES,
    UTILITIES,
    ORGANIZATION,
    TESTING
};

class ModuleInfo
{
public:
    ModuleInfo();
    ModuleInfo(const ModuleId& id, const QString& name);
    ModuleInfo(const ModuleId& id, const QString& name,
               ModuleCategory category);

    ModuleId getId() const;
    void setId(const ModuleId& value);

    QString getModuleName() const;
    void setName(const QString& value);

    std::function<Module*()> getFactoryMethod() const;
    void setFactoryMethod(const std::function<Module*()>& value);

    ModuleCategory getModuleCategory() const;
    void setCategory(const ModuleCategory& value);

private:
    ModuleId id;
    ModuleCategory category = ModuleCategory::HIDDEN;
    QString name;
    std::function<Module*()> factoryMethod;
};
