/*
 * This file is part of Skyward Hub.
 *
 * Skyward Hub is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * Skyward Hub is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * Skyward Hub. If not, see <https://www.gnu.org/licenses/>.
 *
 */

#pragma once

#include <QList>
#include <QMap>
#include <QString>

namespace PayloadStatesList
{

// Rockets states
enum class State : int
{
    INVALID = 0,
    INIT,
    INIT_ERROR,
    INIT_DONE,
    SENSORS_CALIBRATION,
    ALGOS_CALIBRATION,
    DISARMED,
    TEST_MODE,
    ARMED,
    ASCENDING,
    DROGUE_DESCENT,
    WING_DESCENT,
    LANDED,
};

// Groundstation labels
static const QList<QString> statesLabels{
    "INIT",      "CALIBRATE\nSENSORS", "CALIBRATE\nALGORITHMS",
    "DISARMED",  "TEST\nMODE",         "ARMED",
    "ASCENDING", "DROGUE\nDESCENT",    "WING\nDESCENT",
    "LANDED",
};

// Map from rocket states to groundstation labels
static const QMap<State, int> statesIndexes{{State::INVALID, -1},
                                            {State::INIT, 0},
                                            {State::INIT_ERROR, 0},
                                            {State::INIT_DONE, 0},
                                            {State::SENSORS_CALIBRATION, 1},
                                            {State::ALGOS_CALIBRATION, 2},
                                            {State::DISARMED, 3},
                                            {State::TEST_MODE, 4},
                                            {State::ARMED, 5},
                                            {State::ASCENDING, 6},
                                            {State::DROGUE_DESCENT, 7},
                                            {State::WING_DESCENT, 8},
                                            {State::LANDED, 9}};

}  // namespace PayloadStatesList