/*
 * This file is part of Skyward Hub.
 *
 * Skyward Hub is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * Skyward Hub is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * Skyward Hub. If not, see <https://www.gnu.org/licenses/>.
 *
 */

#pragma once

#include <QString>

#include "Core/XmlObject.h"

class ValueElement
{

public:
    ValueElement();
    ~ValueElement();
    ValueElement(const ValueElement& other);
    ValueElement(const QString& name, const QString& topic,
                 const QString& receivedValue, const QString& displayedValue,
                 const QString& color, const QString& outputTopic = "");

    ValueElement operator=(const ValueElement& other);
    QString getName() const;
    void setName(const QString& value);

    QString getTopic() const;
    void setTopic(const QString& value);

    QString getReceivedValue() const;
    void setReceivedValue(const QString& value);

    QString getDisplayedValue() const;
    void setDisplayedValue(const QString& value);

    QString getColor() const;
    void setColor(const QString& value);

    QString toString() const;

    XmlObject toXmlObject() const;
    bool fromXmlObject(const XmlObject& xmlObject);

    QString getCurrentValue() const;
    bool updateCurrentValue(const QString& value);

    QString getOutputTopic() const;
    void setOutputTopic(const QString& value);

    bool isEmitActive() const;

    bool isDisplayInView() const;
    void setDisplayInView(bool value);

protected:
    void copy(const ValueElement& other);

private:
    QString name           = "";
    QString topic          = "";
    QString receivedValue  = "";
    QString displayedValue = "";
    QString color          = "#ffffff";
    QString outputTopic    = "";
    QString currentValue   = "";
    bool displayInView     = true;
};
